extends SSTVEncoder
class_name Martin1

const BLANKING_PULSE_FREQ = 1500.0
const COLOR_FREQ_MULT = 3.1372549

func get_rgb_value_as_freq(image: Image, scan_line: int, vert_pos: int) -> Array:
	#var index = scan_line * (vertResolution * 4) + vert_pos * 4
	var color = image.get_pixel(vert_pos, scan_line)
	var red = float(color.r8) * COLOR_FREQ_MULT + 1500.0
	var green = float(color.g8) * COLOR_FREQ_MULT + 1500.0
	var blue = float(color.b8) * COLOR_FREQ_MULT + 1500.0
	return [red, green, blue]

var numScanLines = 256.0
var vertResolution = 320.0
var blankingInterval = 0.000572
var scanLineLength = 0.146432
var syncPulseLength = 0.004862
var preparedImage = []

func PrepareImage(image: Image):
	image.resize(vertResolution, numScanLines)
	image.convert(Image.FORMAT_RGB8)

	for scanLine in range(0, numScanLines):
		var red = []
		var green = []
		var blue = []
		for vertPos in range(0, vertResolution):
			var freqs = get_rgb_value_as_freq(image, scanLine, vertPos)
			red.push_back(freqs[0]);
			green.push_back(freqs[1]);
			blue.push_back(freqs[2]);
		preparedImage.push_back([green, blue, red]);

func EncodeSSTV(image: Image):
	PrepareImage(image)
	
	EncodePrefix()
	EncodeHeader([false, true, false, true, true, false, false])

	for scanLine in range(0, numScanLines):
		outputFloats.append_array(GenerateTone(SYNC_PULSE_FREQ, syncPulseLength))
		outputFloats.append_array(GenerateTone(BLANKING_PULSE_FREQ, blankingInterval))
		for dataLine in range(0, 3):
			outputFloats.append_array(GenerateToneFromCurve(preparedImage[scanLine][dataLine], scanLineLength))
			outputFloats.append_array(GenerateTone(BLANKING_PULSE_FREQ, blankingInterval))
	
	return outputFloats
