extends SSTVEncoder
class_name RobotBW8

var preparedImage: Array

const BLACK = 1500.0
const WHITE = 2300.0
const COLOR_FREQ_MULT = 3.1372549
const BLANKING_PULSE_FREQ = 1500.0

const SYNC_PULSE = 1200.0

var scanLineLength = 0.056
var syncPulseLength = 0.01
var blankingInterval = 0.000572
var numScanLines = 120

func PrepareImage(image: Image):
	image.resize(160, numScanLines)
	image.convert(Image.FORMAT_RGB8)
	
	for h in range(image.get_height()):
		var line = PackedFloat32Array()
		for w in range(image.get_width()):
			var color = image.get_pixel(w, h)
			line.push_back(round((color.r8 + color.g8 + color.b8) / 3) * COLOR_FREQ_MULT + BLACK)
		preparedImage.push_back(line)

func EncodeSSTV(image: Image):
	PrepareImage(image)
	
	EncodePrefix()
	EncodeHeader([false, false, false, false, false, true, false])
	
	for scanLine in range(0, numScanLines):
		outputFloats.append_array(GenerateTone(SYNC_PULSE_FREQ, syncPulseLength))
		outputFloats.append_array(GenerateTone(BLANKING_PULSE_FREQ, blankingInterval))
		outputFloats.append_array(GenerateToneFromCurve(preparedImage[scanLine], scanLineLength))
		outputFloats.append_array(GenerateTone(BLANKING_PULSE_FREQ, blankingInterval))
		
	return outputFloats
